from osbot_utils.base_classes.Type_Safe import Type_Safe

# todo refactor common logic into this class
class LLM__Platform_Engine(Type_Safe):
    def is_provider_available(self):
        pass

    def execute_request(self):
        pass